Writer

Trait Writer 

pub trait Writer<T> {
    // Required methods
    fn poll(&mut self, item: &T) -> Option<Duration>;
    fn done(&self) -> bool;
}
A simple sink for byte slices.

Required Methods

fn poll(&mut self, item: &T) -> Option<Duration>

Returns an amount of time to wait before retrying, or None for success.

fn done(&self) -> bool

Indicates if the sink has committed all sent data and can be safely dropped.

Dyn Compatibility

This trait is dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety".

Implementors