concatenate

Function concatenate 

pub fn concatenate<'scope, T, C, I>(
    scope: Scope<'scope, T>,
    iterator: I,
) -> Collection<'scope, T, C>
where T: Timestamp, C: Container, I: IntoIterator<Item = Collection<'scope, T, C>>,
Concatenates multiple collections.

This method has the effect of a sequence of calls to concat, but it does so in one operator rather than a chain of many operators.

Examples

use differential_dataflow::input::Input;

::timely::example(|scope| {

    let data = scope.new_collection_from(1 .. 10).1;

    let odds = data.clone().filter(|x| x % 2 == 1);
    let evens = data.clone().filter(|x| x % 2 == 0);

    differential_dataflow::collection::concatenate(scope, vec![odds, evens])
        .assert_eq(data);
});