Specializations of Collection that use Vec as the container.
Type Aliases
- Collection
- An evolving collection of values of type
D, backed by RustVectypes as containers.
Specializations of Collection that use Vec as the container.
D, backed by Rust Vec types as containers.