Module batcher

Module batcher 

A Batcher for columnar streams that merges sorted chains via the free functions in trie_merger.

Callers feed already-chunked, sorted-and-consolidated UpdatesTyped<U> into the batcher via PushInto; forming such chunks from RecordedUpdates<U> is the responsibility of the surrounding dataflow operator’s chunker (TrieChunker).

Structs

MergeBatcher
Creates batches from chunks of sorted, consolidated columnar updates.