Module layout

Module layout 

Layout traits for columnar arrangements.

ColumnarUpdate names the four constituent columnar types of an update, and ColumnarLayout glues them into a DD Layout backed by Coltainer.

Structs

Coltainer
Container, anchored by C to provide an owned type.
ColumnarLayout
A layout based on columnar

Traits

ColumnarUpdate
A type that names constituent update types.
OrdContainer
A container whose references can be ordered.