suffix_chunk

Function suffix_chunk 

pub fn suffix_chunk<U: Update>(
    cursor: (usize, usize, usize),
    batch: &UpdatesTyped<U>,
) -> UpdatesTyped<U>
where U::Time: 'static,
Reconstruct batch[cursor..] as a fresh standalone UpdatesTyped, where cursor is a (key, val, time) index triple into batch’s flat columns.

Used to materialize the unconsumed suffix of a merge survivor: for the Chunk deque protocol, the chunk pushed back to the front of an input deque.