pub struct Present;Expand description
A zero-sized difference that indicates the presence of a record.
This difference type has no negation, and present records cannot be retracted. Addition and multiplication maintain presence, and zero does not inhabit the type.
The primary feature of this type is that it has zero size, which reduces the overhead of differential dataflow’s representations for settings where collections either do not change, or for which records are only added (for example, derived facts in Datalog).