Struct differential_dataflow::operators::arrange::BatchWrapper [−][src]
pub struct BatchWrapper<T> {
pub item: T,
}Wrapper type to permit transfer of Rc types, as in batch.
The BatchWrappers sole purpose in life is to implement Abomonation with methods that panic
when called. This allows the wrapped data to be transited along timely's Pipeline channels.
The wrapper cannot fake out Send, and so cannot be used on timely's Exchange channels, which
is good.
Fields
item: T
The wrapped item.