Module join

Module join 

The proxy join framework.

A conventional differential join against (u64, u64) values, which are provided by and then interpreted by a backend, who is relieved of lattice-time reasoning.

Structs

JoinInstance
A unit of proxied join work, presented to the backend.
ProxyJoinTactic
A proxy-space JoinTactic: matches records of the two presented runs by key_hash,

Traits

ProxyJoinBackend
A type that can interpret and retire pairs of batches, joined by key hashes.