ProxyJoinTactic

Struct ProxyJoinTactic 

pub struct ProxyJoinTactic<B0, B1, Bk> { /* private fields */ }
A proxy-space JoinTactic: matches records of the two presented runs by key_hash,

Implementations

Source

impl<B0, B1, Bk> ProxyJoinTactic<B0, B1, Bk>

pub fn new(backend: Bk) -> Self

A join tactic deferring all value semantics to backend.