Module join

Module join 

Match pairs of records based on a key.

The various join implementations require that the units of each collection can be multiplied, and that the multiplication distributes over addition. That is, we will repeatedly evaluate (a + b) * c as (a * c)

  • (b * c), and if this is not equal to the former term, little is known about the actual output.

Enums

Fresh
Which input contributed the freshly-arrived batch of a deferred join unit.

Traits

JoinTactic
A type that can manage the joining of lists of batches.

Functions

join_traces
An equijoin of two traces, sharing a common key type.
join_with_tactic
Drives an equijoin of two traces using a supplied JoinTactic.