Module chunker

Module chunker 

Organize streams of data into sorted chunks.

Structs

ColumnationChunker
Chunk a stream of vectors into chains of columnation stacks.
ContainerChunker
Chunk a stream of containers into chains of vectors.