Module differential_dataflow::trace::implementations::hash [] [src]

Trace and batch implementations based on Robin Hood hashing.

The types and type aliases in this module start with either

  • HashVal: Collections whose data have the form (key, val) where key is hash-ordered.
  • HashKey: Collections whose data have the form key where key is hash-ordered.

Although HashVal is more general than HashKey, the latter has a simpler representation and should consume fewer resources (computation and memory) when it applies.

Structs

HashKeyBatch

An immutable collection of update tuples, from a contiguous interval of logical times.

HashKeyBuilder

A builder for creating layers from unsorted update tuples.

HashKeyCursor

A cursor for navigating a single layer.

HashValBatch

An immutable collection of update tuples, from a contiguous interval of logical times.

HashValBuilder

A builder for creating layers from unsorted update tuples.

HashValCursor

A cursor for navigating a single layer.

Type Definitions

HashKeySpine

A trace implementation for empty values using a spine of hash-map batches.

HashValSpine

A trace implementation using a spine of hash-map batches.