Module differential_dataflow::trace::implementations::ord
[−]
[src]
Trace and batch implementations based on sorted ranges.
The types and type aliases in this module start with either
OrdVal: Collections whose data have the form(key, val)wherekeyis ordered.OrdKey: Collections whose data have the formkeywherekeyis ordered.
Although OrdVal is more general than OrdKey, the latter has a simpler representation
and should consume fewer resources (computation and memory) when it applies.
Structs
| OrdKeyBatch |
An immutable collection of update tuples, from a contiguous interval of logical times. |
| OrdKeyBuilder |
A builder for creating layers from unsorted update tuples. |
| OrdKeyCursor |
A cursor for navigating a single layer. |
| OrdKeyMerger |
State for an in-progress merge. |
| OrdValBatch |
An immutable collection of update tuples, from a contiguous interval of logical times. |
| OrdValBuilder |
A builder for creating layers from unsorted update tuples. |
| OrdValCursor |
A cursor for navigating a single layer. |
| OrdValMerger |
State for an in-progress merge. |
Type Definitions
| OrdKeySpine |
A trace implementation for empty values using a spine of ordered lists. |
| OrdKeySpineAbom |
A trace implementation for empty values using a spine of abomonated ordered lists. |
| OrdValSpine |
A trace implementation using a spine of ordered lists. |
| OrdValSpineAbom |
A trace implementation using a spine of abomonated ordered lists. |