pub struct OrdKeyBatch<L: Layout> {
pub storage: OrdKeyStorage<L>,
pub description: Description<Time<L>>,
pub updates: usize,
pub value: L::ValContainer,
}Expand description
An immutable collection of update tuples, from a contiguous interval of logical times.
The L parameter captures how the updates should be laid out, and C determines which
merge batcher to select.
Fields
storage: OrdKeyStorage<L>The updates themselves.
description: Description<Time<L>>Description of the update times this layer represents.
updates: usizeThe number of updates reflected in the batch.
We track this separately from storage because due to the singleton optimization,
we may have many more updates than storage.updates.len(). It should equal that
length, plus the number of singleton optimizations employed.
value: L::ValContainerSingle value to return if asked.
Implementations
Sourceimpl<L: Layout<ValContainer: BatchContainer<Owned: Default>>> OrdKeyBatch<L>
impl<L: Layout<ValContainer: BatchContainer<Owned: Default>>> OrdKeyBatch<L>
Sourcepub fn create_value() -> L::ValContainer
pub fn create_value() -> L::ValContainer
Creates a container with one value, to slot in to self.value.