pub struct OrdValBatch<L: Layout> {
pub storage: OrdValStorage<L>,
pub description: Description<Time<L>>,
pub updates: usize,
}Expand description
An immutable collection of update tuples, from a contiguous interval of logical times.
The L parameter captures how the updates should be laid out, and C determines which
merge batcher to select.
Fields
storage: OrdValStorage<L>The updates themselves.
description: Description<Time<L>>Description of the update times this layer represents.
updates: usizeThe number of updates reflected in the batch.
We track this separately from storage because due to the singleton optimization,
we may have many more updates than storage.updates.len(). It should equal that
length, plus the number of singleton optimizations employed.