pub struct OrdValBuilder<L: Layout, CI> {
pub result: OrdValStorage<L>,
/* private fields */
}Expand description
A builder for creating layers from unsorted update tuples.
Fields
result: OrdValStorage<L>The in-progress result.
This is public to allow container implementors to set and inspect their container.