pub fn strongly_connected_at<'scope, T, N, R, F>(
graph: VecCollection<'scope, T, (N, N), R>,
logic: F,
) -> VecCollection<'scope, T, (N, N), R>Expand description
Returns the subset of edges in the same strongly connected component.
This variant introduces node labels in rounds indicated by logic, as in propagate_at:
small labels can complete their propagation before larger labels are introduced, which can
substantially reduce the total work performed.