Types and traits associated with collections of data.
The Collection type is differential dataflow’s core abstraction for an updatable pile of data.
Most differential dataflow programs are “collection-oriented”, in the sense that they transform one collection into another, using operators defined on collections. This contrasts with a more imperative programming style, in which one might iterate through the contents of a collection manually. The higher-level of programming allows differential dataflow to provide efficient implementations, and to support efficient incremental updates to the collections.
Re-exports
pub use vec::Collection as VecCollection;
Modules
- containers
- Traits that can be implemented by containers to provide functionality to collections based on them.
- vec
- Specializations of
Collectionthat useVecas the container.
Structs
- Collection
- An evolving collection represented by a stream of abstract containers.
Traits
- AsCollection
- Conversion to a differential dataflow Collection.
Functions
- concatenate
- Concatenates multiple collections.