pub struct TrieChunker<U: ColumnarUpdate> { /* private fields */ }Expand description
A chunker that unwraps RecordedUpdates into bare UpdatesTyped for the merge batcher.
The intended behavior is to produce chunks whose size is within 1-2x LINK_TARGET.
It ships large batches immediately, accumulates small batches, consolidates as they
exceed 2xLINK_TARGET, and ships them unless they drop below 1xLINK_TARGET.
The flow is into (or around) self.stage, then consolidated blocks into self.ready,
each of which is put in self.stage