pub trait BytesStore {
// Required method
fn store(&mut self, bytes: &[u8]) -> Box<dyn BytesSource>;
}Expand description
Append a chunk’s bytes to backing storage, returning a handle to read them back.
Required Methods
Sourcefn store(&mut self, bytes: &[u8]) -> Box<dyn BytesSource>
fn store(&mut self, bytes: &[u8]) -> Box<dyn BytesSource>
Persist bytes, returning a source that can reproduce them.
Dyn Compatibility
This trait is dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety".