Struct differential_dataflow::difference::DiffPair [−][src]
The difference defined by a pair of difference elements.
This type is essentially a "pair", though in Rust the tuple types do not derive the numeric traits we require, and so we need to emulate the types ourselves. In the interest of ergonomics, we may eventually replace the numeric traits with our own, so that we can implement them for tuples and allow users to ignore details like these.
Fields
element1: R1
The first element in the pair.
element2: R2
The second element in the pair.
Methods
impl<R1: Diff, R2: Diff> DiffPair<R1, R2>[src]
impl<R1: Diff, R2: Diff> DiffPair<R1, R2>Trait Implementations
impl<R1: Copy + Diff, R2: Copy + Diff> Copy for DiffPair<R1, R2>[src]
impl<R1: Copy + Diff, R2: Copy + Diff> Copy for DiffPair<R1, R2>impl<R1: Ord + Diff, R2: Ord + Diff> Ord for DiffPair<R1, R2>[src]
impl<R1: Ord + Diff, R2: Ord + Diff> Ord for DiffPair<R1, R2>fn cmp(&self, other: &DiffPair<R1, R2>) -> Ordering[src]
fn cmp(&self, other: &DiffPair<R1, R2>) -> OrderingThis method returns an Ordering between self and other. Read more
fn max(self, other: Self) -> Self1.21.0[src]
fn max(self, other: Self) -> SelfCompares and returns the maximum of two values. Read more
fn min(self, other: Self) -> Self1.21.0[src]
fn min(self, other: Self) -> SelfCompares and returns the minimum of two values. Read more
impl<R1: PartialOrd + Diff, R2: PartialOrd + Diff> PartialOrd for DiffPair<R1, R2>[src]
impl<R1: PartialOrd + Diff, R2: PartialOrd + Diff> PartialOrd for DiffPair<R1, R2>fn partial_cmp(&self, other: &DiffPair<R1, R2>) -> Option<Ordering>[src]
fn partial_cmp(&self, other: &DiffPair<R1, R2>) -> Option<Ordering>This method returns an ordering between self and other values if one exists. Read more
fn lt(&self, other: &DiffPair<R1, R2>) -> bool[src]
fn lt(&self, other: &DiffPair<R1, R2>) -> boolThis method tests less than (for self and other) and is used by the < operator. Read more
fn le(&self, other: &DiffPair<R1, R2>) -> bool[src]
fn le(&self, other: &DiffPair<R1, R2>) -> boolThis method tests less than or equal to (for self and other) and is used by the <= operator. Read more
fn gt(&self, other: &DiffPair<R1, R2>) -> bool[src]
fn gt(&self, other: &DiffPair<R1, R2>) -> boolThis method tests greater than (for self and other) and is used by the > operator. Read more
fn ge(&self, other: &DiffPair<R1, R2>) -> bool[src]
fn ge(&self, other: &DiffPair<R1, R2>) -> boolThis method tests greater than or equal to (for self and other) and is used by the >= operator. Read more
impl<R1: Eq + Diff, R2: Eq + Diff> Eq for DiffPair<R1, R2>[src]
impl<R1: Eq + Diff, R2: Eq + Diff> Eq for DiffPair<R1, R2>impl<R1: PartialEq + Diff, R2: PartialEq + Diff> PartialEq for DiffPair<R1, R2>[src]
impl<R1: PartialEq + Diff, R2: PartialEq + Diff> PartialEq for DiffPair<R1, R2>fn eq(&self, other: &DiffPair<R1, R2>) -> bool[src]
fn eq(&self, other: &DiffPair<R1, R2>) -> boolThis method tests for self and other values to be equal, and is used by ==. Read more
fn ne(&self, other: &DiffPair<R1, R2>) -> bool[src]
fn ne(&self, other: &DiffPair<R1, R2>) -> boolThis method tests for !=.
impl<R1: Debug + Diff, R2: Debug + Diff> Debug for DiffPair<R1, R2>[src]
impl<R1: Debug + Diff, R2: Debug + Diff> Debug for DiffPair<R1, R2>fn fmt(&self, f: &mut Formatter) -> Result[src]
fn fmt(&self, f: &mut Formatter) -> ResultFormats the value using the given formatter. Read more
impl<R1: Clone + Diff, R2: Clone + Diff> Clone for DiffPair<R1, R2>[src]
impl<R1: Clone + Diff, R2: Clone + Diff> Clone for DiffPair<R1, R2>fn clone(&self) -> DiffPair<R1, R2>[src]
fn clone(&self) -> DiffPair<R1, R2>Returns a copy of the value. Read more
fn clone_from(&mut self, source: &Self)1.0.0[src]
fn clone_from(&mut self, source: &Self)Performs copy-assignment from source. Read more
impl<R1: Diff, R2: Diff> Diff for DiffPair<R1, R2>[src]
impl<R1: Diff, R2: Diff> Diff for DiffPair<R1, R2>fn is_zero(&self) -> bool[src]
fn is_zero(&self) -> boolReturns true if the element is the additive identity. Read more
fn zero() -> Self[src]
fn zero() -> SelfThe additive identity. Read more
impl<R1: Diff, R2: Diff> Add<DiffPair<R1, R2>> for DiffPair<R1, R2>[src]
impl<R1: Diff, R2: Diff> Add<DiffPair<R1, R2>> for DiffPair<R1, R2>type Output = Self
The resulting type after applying the + operator.
fn add(self, rhs: Self) -> Self[src]
fn add(self, rhs: Self) -> SelfPerforms the + operation.
impl<R1: Diff, R2: Diff> Sub<DiffPair<R1, R2>> for DiffPair<R1, R2>[src]
impl<R1: Diff, R2: Diff> Sub<DiffPair<R1, R2>> for DiffPair<R1, R2>type Output = DiffPair<R1, R2>
The resulting type after applying the - operator.
fn sub(self, rhs: Self) -> Self[src]
fn sub(self, rhs: Self) -> SelfPerforms the - operation.
impl<R1: Diff, R2: Diff> Neg for DiffPair<R1, R2>[src]
impl<R1: Diff, R2: Diff> Neg for DiffPair<R1, R2>type Output = Self
The resulting type after applying the - operator.
fn neg(self) -> Self[src]
fn neg(self) -> SelfPerforms the unary - operation.
impl<T: Copy, R1: Diff + Mul<T>, R2: Diff + Mul<T>> Mul<T> for DiffPair<R1, R2> where
<R1 as Mul<T>>::Output: Diff,
<R2 as Mul<T>>::Output: Diff, [src]
impl<T: Copy, R1: Diff + Mul<T>, R2: Diff + Mul<T>> Mul<T> for DiffPair<R1, R2> where
<R1 as Mul<T>>::Output: Diff,
<R2 as Mul<T>>::Output: Diff, type Output = DiffPair<<R1 as Mul<T>>::Output, <R2 as Mul<T>>::Output>
The resulting type after applying the * operator.
fn mul(self, other: T) -> Self::Output[src]
fn mul(self, other: T) -> Self::OutputPerforms the * operation.
impl<R1: Diff, R2: Diff> Abomonation for DiffPair<R1, R2>[src]
impl<R1: Diff, R2: Diff> Abomonation for DiffPair<R1, R2>unsafe fn entomb<W>(&self, _write: &mut W) -> Result<(), Error> where
W: Write, [src]
unsafe fn entomb<W>(&self, _write: &mut W) -> Result<(), Error> where
W: Write, Write any additional information about &self beyond its binary representation. Read more
unsafe fn exhume(&'a mut self, bytes: &'b mut [u8]) -> Option<&'b mut [u8]>[src]
unsafe fn exhume(&'a mut self, bytes: &'b mut [u8]) -> Option<&'b mut [u8]>Recover any information for &mut self not evident from its binary representation. Read more
fn extent(&self) -> usize[src]
fn extent(&self) -> usizeReports the number of further bytes required to entomb self.