Arrange

Trait Arrange 

pub trait Arrange<'scope, T, C>: Sized
where T: Timestamp + Lattice,
{ // Required method fn arrange_named<Ba, Bu, Tr>( self, name: &str, ) -> Arranged<'scope, TraceAgent<Tr>> where Ba: Batcher<Input = C, Time = T> + 'static, Bu: Builder<Time = T, Input = Ba::Output, Output = Tr::Batch>, Tr: Trace<Time = T> + 'static; // Provided method fn arrange<Ba, Bu, Tr>(self) -> Arranged<'scope, TraceAgent<Tr>> where Ba: Batcher<Input = C, Time = T> + 'static, Bu: Builder<Time = T, Input = Ba::Output, Output = Tr::Batch>, Tr: Trace<Time = T> + 'static { ... } }
A type that can be arranged as if a collection of updates.

Required Methods

fn arrange_named<Ba, Bu, Tr>( self, name: &str, ) -> Arranged<'scope, TraceAgent<Tr>>
where Ba: Batcher<Input = C, Time = T> + 'static, Bu: Builder<Time = T, Input = Ba::Output, Output = Tr::Batch>, Tr: Trace<Time = T> + 'static,

Arranges updates into a shared trace, with a supplied name.

Provided Methods

fn arrange<Ba, Bu, Tr>(self) -> Arranged<'scope, TraceAgent<Tr>>
where Ba: Batcher<Input = C, Time = T> + 'static, Bu: Builder<Time = T, Input = Ba::Output, Output = Tr::Batch>, Tr: Trace<Time = T> + 'static,

Arranges updates into a shared trace.

Dyn Compatibility

This trait is not dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety".

Implementors

Source

impl<'scope, T, K, V, R> Arrange<'scope, T, Vec<((K, V), T, R)>> for Collection<'scope, T, (K, V), R>

Source

impl<'scope, T, K: ExchangeData + Hashable, R: ExchangeData + Semigroup> Arrange<'scope, T, Vec<((K, ()), T, R)>> for Collection<'scope, T, K, R>
where T: Timestamp + Lattice + Ord,