Module join

Module join 

Match pairs of records based on a key.

The various join implementations require that the units of each collection can be multiplied, and that the multiplication distributes over addition. That is, we will repeatedly evaluate (a + b) * c as (a * c)

  • (b * c), and if this is not equal to the former term, little is known about the actual output.

Structs

EffortBuilder
A container builder that tracks the length of outputs to estimate the effort of join closures.

Functions

join_traces
An equijoin of two traces, sharing a common key type.

Type Aliases

JoinSession
The session passed to join closures.