Module reduce

Module reduce 

Applies a reduction function on records grouped by key.

The reduce operator acts on (key, val) data. Records with the same key are grouped together, and a user-supplied reduction function is applied to the key and the list of values. The function is expected to populate a list of output values.

Functions

reduce_trace
A key-wise reduction of values in an input trace.