pub struct ChunkBatch<C: Chunk> {
pub chunks: Vec<C>,
pub description: Description<C::Time>,
}Expand description
A batch is a Chunk sequence plus a Description.
Fields
chunks: Vec<C>Ordered, consolidated chunks; their concatenation is the batch.
description: Description<C::Time>The lower, upper, and since frontiers of the batch.
Implementations
Sourceimpl<C: Chunk> ChunkBatch<C>
impl<C: Chunk> ChunkBatch<C>
Sourcepub fn new(chunks: Vec<C>, description: Description<C::Time>) -> Self
pub fn new(chunks: Vec<C>, description: Description<C::Time>) -> Self
Assemble a batch from ordered chunks.