Traits and types for navigating order sequences of update tuples.
The Cursor trait contains several methods for efficiently navigating ordered collections
of tuples of the form (key, val, time, diff). The cursor is different from an iterator
both because it allows navigation on multiple levels (key and val), but also because it
supports efficient seeking (via the seek_key and seek_val methods).
Re-exports
pub use self::cursor_list::CursorList;
Modules
- cursor_
list - A generic cursor implementation merging multiple cursors.
Traits
- Cursor
- A cursor for navigating ordered
(key, val, time, diff)updates. - Navigable
- A batch type that has a cursor for navigation.
Functions
- cursor_
list - Assembles a merged cursor over a sequence of batches.
Type Aliases
- Batch
Cursor - The cursor type for a trace’s batches.
- Batch
Diff - The owned diff type of a trace’s batch cursor.
- Batch
Diff Gat - The borrowed diff type of a trace’s batch cursor.
- Batch
Key - The borrowed key type of a trace’s batch cursor.
- Batch
Time Gat - The borrowed time type of a trace’s batch cursor.
- Batch
Val - The borrowed val type of a trace’s batch cursor.
- Batch
ValOwn - The owned val type of a trace’s batch cursor.