CursorList

Struct CursorList 

pub struct CursorList<C> { /* private fields */ }
Provides a cursor interface over a list of cursors.

The CursorList tracks the indices of cursors with the minimum key, and the the indices of cursors with the minimum key and minimum value. It performs no clever management of these sets otherwise.

Implementations

Source

impl<C: Cursor> CursorList<C>

pub fn new(cursors: Vec<C>, storage: &[C::Storage]) -> Self

Creates a new cursor list from pre-existing cursors.