pub struct CursorList<C> { /* private fields */ }Expand description
Provides a cursor interface over a list of cursors.
The CursorList tracks the indices of cursors with the minimum key, and the the indices of cursors with
the minimum key and minimum value. It performs no clever management of these sets otherwise.
Implementations