Module differential_dataflow::trace::cursor::cursor_pair
[−]
[src]
A generic cursor implementation merging pairs of different cursors.
Structs
| CursorPair |
A cursor over the combined updates of two different cursors. |