pub trait Navigable {
type Cursor: Cursor<Storage = Self>;
// Required method
fn cursor(&self) -> Self::Cursor;
}Expand description
A batch type that has a cursor for navigation.
This is the entry point for accessing batch data through cursors, and the place that opinions
about keys and values are introduced (via the Cursor associated type). Cut-and-merge assembly
is the trace’s concern: TraceReader::batches_through
selects the batches and the defaulted
TraceReader::cursor_through builds a CursorList
over their per-batch cursors.
Required Associated Types
Required Methods
Dyn Compatibility
This trait is dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety".