Organize streams of data into sorted chunks.
Structs
- Columnation
Chunker - Chunk a stream of vectors into chains of columnation stacks.
- Container
Chunker - Chunk a stream of containers into chains of vectors.
Organize streams of data into sorted chunks.