Module differential_dataflow::trace::implementations::ord [] [src]

Trace and batch implementations based on sorted ranges.

The types and type aliases in this module start with either

  • OrdVal: Collections whose data have the form (key, val) where key is ordered.
  • OrdKey: Collections whose data have the form key where key is ordered.

Although OrdVal is more general than OrdKey, the latter has a simpler representation and should consume fewer resources (computation and memory) when it applies.

Structs

OrdKeyBatch

An immutable collection of update tuples, from a contiguous interval of logical times.

OrdKeyBuilder

A builder for creating layers from unsorted update tuples.

OrdKeyCursor

A cursor for navigating a single layer.

OrdValBatch

An immutable collection of update tuples, from a contiguous interval of logical times.

OrdValBuilder

A builder for creating layers from unsorted update tuples.

OrdValCursor

A cursor for navigating a single layer.

Type Definitions

OrdKeySpine

A trace implementation for empty values using a spine of hash-map batches.

OrdValSpine

A trace implementation using a spine of hash-map batches.