pub trait Batch: BatchReader + Sized {
type Merger: Merger<Self>;
// Required method
fn empty(lower: Antichain<Self::Time>, upper: Antichain<Self::Time>) -> Self;
// Provided method
fn begin_merge(
&self,
other: &Self,
compaction_frontier: AntichainRef<'_, Self::Time>,
) -> Self::Merger { ... }
}Expand description
An immutable collection of updates.
Required Associated Types
Required Methods
Provided Methods
Sourcefn begin_merge(
&self,
other: &Self,
compaction_frontier: AntichainRef<'_, Self::Time>,
) -> Self::Merger
fn begin_merge( &self, other: &Self, compaction_frontier: AntichainRef<'_, Self::Time>, ) -> Self::Merger
Initiates the merging of consecutive batches.
The result of this method can be exercised to eventually produce the same result
that a call to self.merge(other) would produce, but it can be done in a measured
fashion. This can help to avoid latency spikes where a large merge needs to happen.
Dyn Compatibility
This trait is not dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety".