pub trait BatchReader: Sized {
type Time: Timestamp + Lattice;
// Required methods
fn len(&self) -> usize;
fn description(&self) -> &Description<Self::Time>;
// Provided methods
fn is_empty(&self) -> bool { ... }
fn lower(&self) -> &Antichain<Self::Time> { ... }
fn upper(&self) -> &Antichain<Self::Time> { ... }
}Expand description
A batch of updates whose contents may be read.
This is a restricted interface to batches of updates, which support the reading of the batch’s contents, but do not expose ways to construct the batches. This trait is appropriate for views of the batch, and is especially useful for views derived from other sources in ways that prevent the construction of batches from the type of data in the view (for example, filtered views, or views with extended time coordinates).
Required Associated Types
Required Methods
Sourcefn description(&self) -> &Description<Self::Time>
fn description(&self) -> &Description<Self::Time>
Describes the times of the updates in the batch.
Provided Methods
Dyn Compatibility
This trait is not dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety".
Implementations on Foreign Types
Sourceimpl<B: BatchReader> BatchReader for Rc<B>
impl<B: BatchReader> BatchReader for Rc<B>
Sourcefn description(&self) -> &Description<Self::Time>
fn description(&self) -> &Description<Self::Time>
Describes the times of the updates in the batch.